#!/bin/bash
#
# This application looks for IPs at log files (/var/log/auth.log and
# /var/log/syslog) that is testing a dictionary attack over POP3, SSHd
#  and FTP  services. On event, it blocks the access of that (or
# those IPs ajusting iptables rules.
#
# How it works?
# For each period it reads the log files looking for malicious
# action, considering the latest IND seconds or the time defined by
# first argument. Then it sleeps for that time and retries again.
#
# When it finds out some malicious action it register the IP at file
# At end of searching, it sorts the file to cut repeated IPs and
# read the file, defining the generic iptables rule
# iptables -I INPUT --source IP -j DROP
#
# This application is free. You may copy, modify and distribute since
# you keep the credits of the author and the URL of GURI website.
# The software is provided "as is" and its author or company is not
# responsable at any kind for damage or losing data.
#
# Credits:
# GURI SoftHouse Ltda
# SOftware Development Division
# Ulisses Thadeu Vieira Guedes
# uli1958@hotmail.com
#
# Release Notes
# version 1.1.2 - Aug, 24, 2013
# Version 1.0.0.- RC3 - Aug, 03, 2010
# Version 1.0.0 - RC2 - Aug, 04,2009
#   Includes a lista of whitelist-host. Errors from those hosts will not be considered.
#   since they are considered thrusted hosts.
# Version 1.0.0 - RC1 - Aug, 28, 2008
# 
##################################################################

function Registra()
{
    local -i IND
    local BUSCA
    local IPS=""
    local IPO=""
    local JA
    local -i SZBLOCKFILE

    
#defining the dates to be inspected of auth.log ..."
    if (( SLP != 0 )) ; then
	IND=$SLP
	BUSCA="`date --date "$IND seconds ago" '+%b %_d %H:%M:%S'`"
	
#	updating the counter
	IND=$IND-1

#	creating the string with dates to be searched
	while [ "$IND" != "0" ];
	do
	{
		BUSCA="$BUSCA|`date --date "$IND seconds ago" '+%b %_d %H:%M:%S'`"
		IND=$IND-1
	} done

#	completing the string with the current date
	HOJE=`date '+%b %_d %H:%M:%S'`
	BUSCA="$BUSCA|$HOJE"
    else
	BUSCA="$HOJE"
	echo "Manual Check at  $HOJE" >> $LOGFILE
    fi
#	starting the search for dictionary attack over ssh
    egrep "$BUSCA" $AUTLOG | \
    egrep  "Failed password for" | \
    awk -F "from" '{ print $2 }' | \
    awk -F "port" '{print $1}' | \
    tr -d ' ' > $BLOCKFILE

# 	seaching for scan over ssh
    egrep "$BUSCA" $AUTLOG | \
    egrep  "Did not receive identification string from" | \
    awk -F "from" '{ print $2 }' | \
    tr -d ' ' >> $BLOCKFILE

#	searching for pop3 scanning like this
#Aug 29 22:04:08 torrinha ipop3d[7888]: Login failed user=root auth=root host=200-101-77-99.bsace705.e.brasiltelecom.net.br [200.101.77.99]
# You MUST adapt the searching key.
    egrep "$BUSCA" $AUTLOG | \
    egrep  -i "pop3d.* Login failed" | \
    awk -F "ip=" '{ print $2 }' | \
    awk -F "ff:" '{ print $2 }' | \
    tr -d ']' >> $BLOCKFILE

#	seaching for ftp under xinetd
#Oct 12 07:56:57 torrinha xinetd[11295]: FAIL: ftp address from=207.58.177.94
    egrep "$BUSCA" $AUTLOG | \
    egrep  "FAIL: ftp address" | \
    awk -F "from=" '{ print $2 }' | \
    tr -d ' ' >> $BLOCKFILE

    SZBLOCKFILE=`stat --format="%s" $BLOCKFILE`
    if (( SZBLOCKFILE == 0 )) ; then
#	echo "$BUSCA not found" >> $LOGFILE
	return;
    fi
    if [ -f $BLOCKFILE ] ; then

    	sort -u < $BLOCKFILE > $BLKFILESR

    	rm -f $BLOCKFILE

	if [ -f $INSPECWHTLST ] ; then

        	while read -e IPS ;
		do {
    			JA="`grep $IPS $INSPECWHTLST`"
    			if [ "$JA" != "$IPS" ]; then
        			if [ "$IPS" != "$IPO" ] ; then
					JA=`/sbin/iptables -L -n | grep $IPS`
	    				if [ "$JA" == "" ]; then
    						/sbin/iptables -A INPUT --source $IPS -j DROP
    						/sbin/iptables-save > $IPTCFG
    						echo "$HOJE : IP [$IPS] was included and IPTABLES updated" >> $LOGFILE
    						UPDLIST="1"
    					fi
        			fi
			fi
			IPO="$IPS"

        	} done < $BLKFILESR

        else
        	while read -e IPS ;
		do {
    			if [ "$IPS" != "$IPO" ] ; then
				JA=`/sbin/iptables -L -n | grep $IPS`
    				if [ "$JA" == "" ]; then
					/sbin/iptables -A INPUT --source $IPS -j DROP
					/sbin/iptables-save > $IPTCFG
					echo "$HOJE : IP [$IPS] was included and IPTABLES updated" >> $LOGFILE
					UPDLIST="1"
				fi
    			fi
			IPO="$IPS"

        	} done < $BLKFILESR
	fi

#	rm -f $BLKFILESR

    fi
}

function Usando()
{
    echo "
GURI SoftHouse Ltda

Inspector.sh  Copyright (C) 2013  Ulisses Guedes

This program comes with ABSOLUTELY NO WARRANTY; 
This is free software, and you are welcome to redistribute it
    
    Inspector.sh version 0.2.2 - Aug 2013

Usage:

Interactive Mode:

Inspector.sh all \"date string\"

It inspects /var/log/auth.log by date string time supplied by \"date string\"
and drops the IPs found doing malicous action using iptables DROP rule

Examples:
Inspector.sh all \"Oct  1\"  (searches by all events at date "Oct  1"
Inspector.sh all \"Oct\"  (searches by all events at month October)
Inspector.sh all \"Oct  1 13\"  (searches by all events at date "Oct  1"
    and hour 13.
    
Inspector.sh all \"date string\" ./local-auth.log
The same as above but using the file local-auth.log at current directory


Batch mode:
Inspector.sh SEC &

It inspects /var/log/auth.log for each SEC seconds starting at current time
and drops the IPs found doing malicous action using iptables DROP rule

To keep it running (even after logout), complete with the bash command disown.

Examples:
[cursor] Inspector.sh 10 &
[1] 12345
[cursor] disown -h 12345

if SEC is "-" it assumes the default sleep time (2 seconds).

If you want configure another sleep time under service mode (/etc/init.d/inspector)
then update that source code or replace the \"-\" by the desired sleep time.
"
}






# SLP is the amount of time o sleep or to search
declare -i SLP
# AUTOLOG is the full path name of auth log file
declare AUTLOG 
# BLOCKFILE is the full path of temporary file with IPs to be blocked
declare BLOCKFILE
# BLKFILESR is the full path of temporary file with IPs ordered to be blocked
declare BLKFILESR
# LOGFILE is the full path of file with Inspector actions
declare LOGFILE
# IPTCFG is the file with current iptables rules
declare IPTCFG
# IPTABLES defines the iptables file 
declare IPTABLE
# INSPECWHTLST is the full path where the whitelist IPs are stored
declare INSPECWHTLST
# defining the full path of auth.log
# It may different at Linux distribution
if [ "$3" == "" ] ; then
	AUTLOG=/var/log/auth.log /var/log/syslog
else
	AUTLOG="$3"
fi

#defining the full path of temporary file
BLOCKFILE=/var/tmp/block
BLKFILESR=/var/tmp/block.sor
LOGFILE=/var/log/Inspector.log
IPTCFG=/etc/sysconfig/iptables
IPTABLE=/sbin/iptables
INSPECWHTLST=/etc/Inspector.whtlist
UPDLIST="0"
UPDLSTF=/var/www/html/inspector/listaIp.txt
#Testing the second argument first
# Testing the first argument
case "$1" in
	-h|help|-help)
	    Usando
	    exit 0
	    ;;
	all) HOJE="$2"
	   SLP=0;
	   Registra
		;;
	*) if [ "$1" == "-" ] ; then
# defining the default time (2 seconds)
		SLP=2
	  else
# using the time defined by user
		SLP=$1
	  fi
	  if  [ "$UPDLSTF" == "" ] ; then
		for ((;;)) ;
		do {
			UPDLIST="0"
			Registra
			sleep $SLP
		} done
	  else
		for ((;;)) ;
		do {
			UPDLIST="0"
			Registra
			if [ "$UPDLIST" == "1" ] ; then
			grep "\[" /var/log/Inspector.log | \
				awk -F "[" '{print $2}' | \
				awk -F "]" '{print $1}' | \
				sort -i > $UPDLSTF
			fi
			sleep $SLP
		} done
	  fi 
	  ;;
esac

